# Generated file
# - do not overwrite
# - do not include in git
from collections import namedtuple

BuildInfo = namedtuple(
    'BuildInfo',
    ['version', 'built_at', 'sha', 'branch', 'tag'],
)

BUILD_INFO = BuildInfo(
    version='v3.24.0~de1a777',
    built_at='2023-05-31 09:40:30Z',
    sha='de1a777fb9287f666e161e98200054ae757fa44c',
    branch='HEAD',
    tag='v3.24.0',
)



import pathlib
from typing import Optional

from .logging import prepare_logging, LOG_FILTER


def _config_to_string(config: object):
    lines = [f'{type(config).__name__}']
    fields = (f for f in config.__dict__.keys() if not f.startswith('_'))
    for field in fields:
        v = str(getattr(config, field))
        t = type(getattr(config, field)).__name__
        lines.append(f'- {field} = {v} [{t}]')
    return '\n'.join(lines)


class ConfigModel:

    def __str__(self):
        return _config_to_string(self)


class GeneralConfig(ConfigModel):

    def __init__(self, environment: str, client_url: str, secret: str):
        self.environment = environment
        self.client_url = client_url
        self.secret = secret


class SentryConfig(ConfigModel):

    def __init__(self, enabled: bool, workers_dsn: Optional[str],
                 traces_sample_rate: Optional[float], max_breadcrumbs: Optional[int]):
        self.enabled = enabled
        self.workers_dsn = workers_dsn
        self.traces_sample_rate = traces_sample_rate
        self.max_breadcrumbs = max_breadcrumbs


class DatabaseConfig(ConfigModel):

    def __init__(self, connection_string: str, connection_timeout: int, queue_timout: int):
        self.connection_string = connection_string
        self.connection_timeout = connection_timeout
        self.queue_timout = queue_timout


class S3Config(ConfigModel):

    def __init__(self, url: str, username: str, password: str,
                 bucket: str, region: str):
        self.url = url
        self.username = username
        self.password = password
        self.bucket = bucket
        self.region = region


class LoggingConfig(ConfigModel):

    def __init__(self, level: str, global_level: str, message_format: str,
                 dict_config: Optional[dict] = None):
        self.level = level
        self.global_level = global_level
        self.message_format = message_format
        self.dict_config = dict_config

    def apply(self):
        prepare_logging(self)

    @staticmethod
    def set_logging_extra(key: str, value: str):
        LOG_FILTER.set_extra(key, value)


class CloudConfig(ConfigModel):

    def __init__(self, multi_tenant: bool):
        self.multi_tenant = multi_tenant


class MailConfig(ConfigModel):

    def __init__(self, enabled: bool, ssl: Optional[bool], name: str, email: str,
                 host: str, port: Optional[int], security: Optional[str],
                 auth: Optional[bool], username: Optional[str],
                 password: Optional[str], rate_limit_window: int,
                 rate_limit_count: int, timeout: int,
                 dkim_selector: Optional[str] = None,
                 dkim_privkey_file: Optional[str] = None):
        self.enabled = enabled
        self.name = name
        self.email = email
        self.host = host
        self.security = 'plain'
        if security is not None:
            self.security = security.lower()
        elif ssl is not None:
            self.security = 'ssl' if ssl else 'plain'
        self.port = port or self._default_port()
        self.auth = auth or (username is not None and password is not None)
        self.username = username if self.auth is not None else None
        self.password = password if self.auth is not None else None
        self.rate_limit_window = rate_limit_window
        self.rate_limit_count = rate_limit_count
        self.timeout = timeout
        self.dkim_selector = dkim_selector
        self.dkim_privkey_file = dkim_privkey_file
        self.dkim_privkey = b''

    def load_dkim_privkey(self):
        if self.dkim_privkey_file is not None:
            self.dkim_privkey = pathlib.Path(self.dkim_privkey_file).read_bytes()
            self.dkim_privkey = self.dkim_privkey.replace(b'\r\n', b'\n')

    @property
    def use_dkim(self):
        return self.dkim_selector is not None and len(self.dkim_privkey) > 0

    @property
    def login_user(self) -> str:
        return self.username or ''

    @property
    def login_password(self) -> str:
        return self.password or ''

    @property
    def is_plain(self):
        return self.security == 'plain'

    @property
    def is_ssl(self):
        return self.security == 'ssl'

    @property
    def is_tls(self):
        return self.security == 'starttls' or self.security == 'tls'

    def _default_port(self) -> int:
        if self.is_plain:
            return 25
        if self.is_ssl:
            return 465
        return 587

    def has_credentials(self) -> bool:
        return self.username is not None and self.password is not None

    def __str__(self):
        return f'MailConfig\n' \
               f'- enabled = {self.enabled}\n' \
               f'- name = {self.name}\n' \
               f'- email = {self.email}\n' \
               f'- host = {self.host}\n' \
               f'- port = {self.port}\n' \
               f'- security = {self.security}\n' \
               f'- auth = {self.auth}\n' \
               f'- rate_limit_window = {self.rate_limit_window}\n' \
               f'- rate_limit_count = {self.rate_limit_count}\n' \
               f'- timeout = {self.timeout}\n' \
               f'- dkim_selector = {self.dkim_selector}\n' \
               f'- dkim_privkey_file = {self.dkim_privkey_file}\n'



import collections

from typing import Any, Optional, Generic, TypeVar, Callable


T = TypeVar('T')


def cast_bool(value: Any) -> bool:
    return bool(value)


def cast_int(value: Any) -> int:
    return int(value)


def cast_optional_int(value: Any) -> Optional[int]:
    if value is None:
        return None
    return int(value)


def cast_float(value: Any) -> float:
    return float(value)


def cast_optional_float(value: Any) -> Optional[float]:
    if value is None:
        return None
    return float(value)


def cast_str(value: Any) -> str:
    return str(value)


def cast_optional_str(value: Any) -> Optional[str]:
    if value is None:
        return None
    return str(value)


def cast_optional_dict(value: Any) -> Optional[dict]:
    if not isinstance(value, dict):
        return None
    return value


class ConfigKey(Generic[T]):

    def __init__(self, yaml_path: list[str], cast: Callable[[Any], T],
                 var_names=None, default=None, required=False):
        self.yaml_path = yaml_path
        self.var_names = var_names or []  # type: list[str]
        self.default = default
        self.required = required
        self.cast = cast

    def __str__(self):
        return 'ConfigKey: ' + '.'.join(self.yaml_path)


class ConfigKeysMeta(type):

    @classmethod
    def __prepare__(mcs, name, bases, **kwargs):
        return collections.OrderedDict()

    def __init__(cls, name, bases, namespace):
        cls._config_keys = []
        for attr in namespace:
            if attr.startswith('_'):
                continue
            value = getattr(cls, attr)
            if isinstance(value, ConfigKey):
                cls._config_keys.append(value)
            if hasattr(value, '_config_keys'):
                keys = getattr(value, '_config_keys')
                if isinstance(keys, list):
                    cls._config_keys.extend(keys)
        super().__init__(name, bases, namespace)

    def __iter__(cls):
        return iter(cls._config_keys)


class ConfigKeysContainer(metaclass=ConfigKeysMeta):
    pass


class _GeneralKeys(ConfigKeysContainer):
    environment = ConfigKey(
        yaml_path=['general', 'environment'],
        var_names=['GENERAL_ENVIRONMENT'],
        default='Production',
        cast=cast_str,
    )
    client_url = ConfigKey(
        yaml_path=['general', 'clientUrl'],
        var_names=['GENERAL_CLIENT_URL'],
        default='http://localhost:8080',
        cast=cast_str,
    )
    secret = ConfigKey(
        yaml_path=['general', 'secret'],
        var_names=['GENERAL_SECRET'],
        default='',
        cast=cast_str,
    )


class _LoggingKeys(ConfigKeysContainer):
    level = ConfigKey(
        yaml_path=['logging', 'level'],
        var_names=['LOGGING_ENVIRONMENT'],
        default='INFO',
        cast=cast_str,
    )
    global_level = ConfigKey(
        yaml_path=['logging', 'globalLevel'],
        var_names=['LOGGING_CLIENT_URL'],
        default='WARNING',
        cast=cast_str,
    )
    format = ConfigKey(
        yaml_path=['logging', 'format'],
        var_names=['LOGGING_FORMAT'],
        default='%(asctime)s | %(levelname)8s | %(name)s: [T:%(traceId)s] %(message)s',
        cast=cast_str,
    )
    dict_config = ConfigKey(
        yaml_path=['logging', 'dict_config'],
        var_names=[],
        default=None,
        cast=cast_optional_dict,
    )


class _CloudKeys(ConfigKeysContainer):
    enabled = ConfigKey(
        yaml_path=['cloud', 'enabled'],
        var_names=['CLOUD_ENABLED'],
        default=False,
        cast=cast_bool,
    )


class _SentryKeys(ConfigKeysContainer):
    enabled = ConfigKey(
        yaml_path=['sentry', 'enabled'],
        var_names=['SENTRY_ENABLED'],
        default=False,
        cast=cast_bool,
    )
    worker_dsn = ConfigKey(
        yaml_path=['sentry', 'workersDsn'],
        var_names=['SENTRY_WORKER_DSN', 'SENTRY_DSN'],
        default='',
        cast=cast_str,
    )
    traces_sample_rate = ConfigKey(
        yaml_path=['sentry', 'tracesSampleRate'],
        var_names=['SENTRY_TRACES_SAMPLE_RATE'],
        default='',
        cast=cast_str,
    )
    max_breadcrumbs = ConfigKey(
        yaml_path=['sentry', 'maxBreadcrumbs'],
        var_names=['SENTRY_MAX_BREADCRUMBS'],
        default='',
        cast=cast_str,
    )


class _DatabaseKeys(ConfigKeysContainer):
    connection_string = ConfigKey(
        yaml_path=['database', 'connectionString'],
        var_names=['DATABASE_CONNECTION_STRING'],
        default='postgresql://postgres:postgres@postgres:5432/engine-wizard',
        cast=cast_str,
    )
    connection_timeout = ConfigKey(
        yaml_path=['database', 'connectionTimeout'],
        var_names=['DATABASE_CONNECTION_TIMEOUT'],
        default=30000,
        cast=cast_int,
    )
    queue_timeout = ConfigKey(
        yaml_path=['database', 'queueTimeout'],
        var_names=['DATABASE_QUEUE_TIMEOUT'],
        default=180,
        cast=cast_int,
    )


class _S3Keys(ConfigKeysContainer):
    url = ConfigKey(
        yaml_path=['s3', 'url'],
        var_names=['S3_URL'],
        default='http://minio:9000',
        cast=cast_str,
    )
    bucket = ConfigKey(
        yaml_path=['s3', 'bucket'],
        var_names=['S3_BUCKET'],
        default='engine-wizard',
        cast=cast_str,
    )
    region = ConfigKey(
        yaml_path=['s3', 'region'],
        var_names=['S3_REGION'],
        default='eu-central-1',
        cast=cast_str,
    )
    username = ConfigKey(
        yaml_path=['s3', 'username'],
        var_names=['S3_USERNAME'],
        default='minio',
        cast=cast_str,
    )
    password = ConfigKey(
        yaml_path=['s3', 'password'],
        var_names=['S3_PASSWORD'],
        default='minioPassword',
        cast=cast_str,
    )


class _MailKeys(ConfigKeysContainer):
    enabled = ConfigKey(
        yaml_path=['mail', 'enabled'],
        var_names=['MAIL_ENABLED'],
        default=True,
        cast=cast_bool,
    )
    name = ConfigKey(
        yaml_path=['mail', 'name'],
        var_names=['MAIL_NAME'],
        default='',
        cast=cast_str,
    )
    email = ConfigKey(
        yaml_path=['mail', 'email'],
        var_names=['MAIL_EMAIL'],
        default='',
        cast=cast_str,
    )
    host = ConfigKey(
        yaml_path=['mail', 'host'],
        var_names=['MAIL_HOST'],
        default='',
        cast=cast_str,
    )
    port = ConfigKey(
        yaml_path=['mail', 'port'],
        var_names=['MAIL_PORT'],
        cast=cast_str,
    )
    ssl = ConfigKey(
        yaml_path=['mail', 'ssl'],
        var_names=[],
        cast=cast_optional_str,
    )
    security = ConfigKey(
        yaml_path=['mail', 'security'],
        var_names=['MAIL_SECURITY'],
        cast=cast_optional_str,
    )
    auth_enabled = ConfigKey(
        yaml_path=['mail', 'authEnabled'],
        var_names=[],
        default=False,
        cast=cast_bool,
    )
    username = ConfigKey(
        yaml_path=['mail', 'username'],
        var_names=['MAIL_USERNAME'],
        cast=cast_str,
    )
    password = ConfigKey(
        yaml_path=['mail', 'password'],
        var_names=['MAIL_PASSWORD'],
        cast=cast_str,
    )
    rate_limit_window = ConfigKey(
        yaml_path=['mail', 'rateLimit', 'window'],
        var_names=['MAIL_RATE_LIMIT_WINDOW'],
        default=0,
        cast=cast_int,
    )
    rate_limit_count = ConfigKey(
        yaml_path=['mail', 'rateLimit', 'count'],
        var_names=['MAIL_RATE_LIMIT_COUNT'],
        default=0,
        cast=cast_int,
    )
    timeout = ConfigKey(
        yaml_path=['mail', 'timeout'],
        var_names=['MAIL_TIMEOUT'],
        default=10,
        cast=cast_int,
    )
    dkim_selector = ConfigKey(
        yaml_path=['mail', 'dkim', 'selector'],
        var_names=['MAIL_DKIM_SELECTOR'],
        default=None,
        cast=cast_optional_str,
    )
    dkim_privkey_file = ConfigKey(
        yaml_path=['mail', 'dkim', 'privkey_file'],
        var_names=['MAIL_DKIM_PRIVKEY_FILE'],
        default=None,
        cast=cast_optional_str,
    )


class ConfigKeys(ConfigKeysContainer):
    cloud = _CloudKeys
    database = _DatabaseKeys
    general = _GeneralKeys
    logging = _LoggingKeys
    mail = _MailKeys
    s3 = _S3Keys
    sentry = _SentryKeys


if __name__ == '__main__':
    for key in ConfigKeys:
        print(str(key))



import logging
import logging.config
import sys


class DSWLogFilter(logging.Filter):

    def __init__(self, extras=None):
        super().__init__()
        self.extras = extras or {'traceId': ''}

    def set_extra(self, key: str, value: str):
        self.extras[key] = value

    def filter(self, record: logging.LogRecord):
        record.__dict__.update(self.extras)
        return True


LOG_FILTER = DSWLogFilter()


class DSWLogger(logging.Logger):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.addFilter(LOG_FILTER)


def prepare_logging(logging_cfg):
    if logging_cfg.dict_config is not None:
        logging.config.dictConfig(logging_cfg.dict_config)
    else:
        logging.basicConfig(
            stream=sys.stdout,
            level=logging_cfg.global_level,
            format=logging_cfg.message_format
        )
        dsw_loggers = (logging.getLogger(name)
                       for name in logging.root.manager.loggerDict.keys()
                       if name.lower().startswith('dsw'))
        for logger in dsw_loggers:
            logger.setLevel(logging_cfg.level)
    # Set for all existing loggers
    logging.getLogger().addFilter(filter=LOG_FILTER)
    loggers = (logging.getLogger(name)
               for name in logging.root.manager.loggerDict.keys())
    for logger in loggers:
        logger.addFilter(filter=LOG_FILTER)
    # Set for any future loggers
    logging.setLoggerClass(DSWLogger)
